#!/usr/bin/env python

import subprocess
import json
import os
import csv
import re

# Maximally allow rw-r--r-- permissions on new files
os.umask(0o133)

# Inverse of letter, numbers, and hyphen used to sanitize strings
CHAR_BLACKLIST_RE = re.compile('[^a-zA-Z0-9-]')

def decode_and_sanitize_string(st):
    st = st.decode("ASCII")
    st = st.strip()
    st = CHAR_BLACKLIST_RE.sub("-", st)
    return st

# Recursively search the arch tree for the given arch.
# Returns an ordered list of arches between the search item and root.
def search_arch_tree(tree, arch):
        if not 'arches' in tree.keys():
                return []
        elif arch in tree['arches'].keys():
                return [ arch ]
        else:
                for a in tree['arches'].keys():
                        l = search_arch_tree(tree['arches'][a], arch)
                        if len(l) > 0:
                                l.append(a)
                                return l
                return []

# Get DeviceCode and TargetClass using fw_printenv
try:
    if subprocess.check_output(["mount"]).find(b"/boot") < 0:
        subprocess.check_call(["mount", "/boot"])
except subprocess.CalledProcessError as e:
    print("ERROR: unable to mount /boot")
    exit(1)

try:
    deviceCode = subprocess.check_output(["fw_printenv", "-n", "DeviceCode"])
    targetClass = subprocess.check_output(["fw_printenv", "-n", "TargetClass"])
except OSError as e:
    print("ERROR: fw_printenv was not found")
    exit(1)
except subprocess.CalledProcessError as e:
    print("ERROR: fw_printenv returned an error")
    exit(1)

# Post-process the values a bit
try:
    deviceCode = decode_and_sanitize_string(deviceCode)[2:].upper()

    targetClass = decode_and_sanitize_string(targetClass)

    # XXX treat USRPs like PXIs since software stack is very similar
    if targetClass == "USRP-Stand-Alone-Devices":
        targetClass = "PXI"
except IndexError as e:
    print("ERROR: Invalid values for DeviceCode('%s') or TargetClass('%s')" % (deviceCode, targetClass))
    exit(1)

# Get highest priority value and noarch priority value from existing arch.conf
prio_high = -1
prio_arch = -1
arch_str = ""
try:
    with open("/etc/opkg/arch.conf") as f:
        for x in f:
            prio = int(x.split(' ')[2])
            arch = x.split(' ')[1]
            if arch == 'noarch':
                prio_noarch = prio
            if prio > prio_high:
                prio_high = prio
            if len(arch_str) == 0:
                if x.find('arch arm') >= 0:
                    arch_str = "ARM"
                elif x.find('arch x64') >= 0:
                    arch_str = "x64"
except EnvironmentError:
    print("ERROR: Could not access arch.conf")
    exit(1)
except IndexError:
    print("ERROR: invalid entry found in arch.conf")
    exit(1)

# Read in device database and arch tree and construct a map
# of PIDs to arch list
pidmap = {}
try:
    currentDir = os.path.dirname(os.path.realpath(__file__))
    with open(os.path.join(currentDir, "arch.json")) as archfile:
        arch_tree = json.load(archfile)
        with open(os.path.join(currentDir, "TargetIDArches.csv")) as csvfile:
            pids = csv.DictReader(csvfile)
            for row in pids:
                l = search_arch_tree(arch_tree, row['ARCHITECTURE'])
                l.reverse()
                # remove arch value from pidmap so we don't end up with an "ARM" or "x64" arch
                if arch_str in l:
                    l.remove(arch_str)
                pidmap[row['PROCESSOR']] = l
except:
    print("ERROR: Unable to load arch.json or TargetIDArches.csv")
    exit(1)

# Generate new arch definitions
# Generate temp file then rename to ensure atomicity w.r.t to opkg
prio_cntr = prio_high + 5
try:
    if os.path.exists("/etc/opkg/ni-arch.conf.new"):
        os.remove("/etc/opkg/ni-arch.conf.new")
    with open("/etc/opkg/ni-arch.conf.new", "w") as f:
        f.write("arch %s %d\n" % (targetClass, prio_noarch+2))
        if deviceCode in pidmap.keys():
            for d in pidmap[deviceCode]:
                f.write("arch %s %d\n" % (d, prio_cntr))
                prio_cntr += 1
        f.write("arch DeviceCode%s %d\n" % (deviceCode, prio_cntr))
    os.rename("/etc/opkg/ni-arch.conf.new", "/etc/opkg/ni-arch.conf")
except:
    print("ERROR: Unable to generate ni-arch.conf")
    exit(1)

